#include "device.h"
#include "utils/array.h"
#include "usb/usb.h"
#include <dlfcn.h>
#include <string.h>
#include <stdio.h>

struct DEVICE
{
    // Device driver
    void* driver;

    // Physical device handle
    void* handle;
};

array_t* device_array;

void device_init()
{
    device_array = array_alloc();
    usb_init();
}

array_t* device_get_array()
{
    return device_array;
}

device_t* device_register(void* driver, void* handle)
{
    device_t* tr = malloc(sizeof(device_t));
    tr->driver = driver;
    tr->handle = handle;

    array_add(device_array, tr);

    return tr;
}

char* device_get_image(device_t* device)
{
    char* (*getimage_fn)(void*) = dlsym(device->driver, "driver_get_image");
    if(!getimage_fn)
    {
        fprintf(stderr, "Warning: driver does not implement `driver_get_image`\n");
        char* tr = malloc(1);
        *tr = 0;
        return tr;
    }

    char* image = getimage_fn(device->handle);
    size_t image_len = strlen(image);
    
    // TODO : Change PATH so that 'build' becomes executable directory
    char* path = "build/drivers/assets/";
    size_t path_len = strlen(path);

    char* tr = malloc(image_len + path_len + 1);
    memcpy(tr, path, path_len);
    memcpy(tr + path_len, image, image_len);
    *(tr + path_len + image_len) = 0;

    return tr;
}

char* device_get_name(device_t* device)
{
    char* (*getname_fn)(void*) = dlsym(device->driver, "driver_get_name");
    if(!getname_fn)
    {
        fprintf(stderr, "Warning: driver does not implement `driver_get_name`\n");
        return "";
    }

    return getname_fn(device->handle);
}

device_capacity_t device_get_capacity(device_t* device)
{
    device_capacity_t (*getcapacity_fn)(void*) = dlsym(device->driver, "driver_get_capacity");
    if(!getcapacity_fn)
    {
        fprintf(stderr, "Warning: driver does not implement `driver_get_capacity`\n");
        return 0;
    }

    return getcapacity_fn(device->handle);
}

char* device_get_manufacturer(device_t* device)
{
    char* (*getmanufacturer_fn)(void*) = dlsym(device->driver, "driver_get_manufacturer");
    if(!getmanufacturer_fn)
    {
        fprintf(stderr, "Warning: driver does not implement `driver_get_manufacturer`\n");
        return "";
    }
    return getmanufacturer_fn(device->handle);
}

int device_mouse_get(device_t* device, struct MOUSE* output)
{
    int (*mouseget_fn)(void*, struct MOUSE*) = dlsym(device->driver, "driver_mouse_get");
    if(!mouseget_fn)
    {
        fprintf(stderr, "Warning: driver does not implement `driver_mouse_get`\n");
        return -1;
    }

    return mouseget_fn(device->handle, output);
}

int device_wireless_get(device_t* device, struct WIRELESS* output)
{
    int (*wirelessget_fn)(void*, struct WIRELESS*) = dlsym(device->driver, "driver_wireless_get");
    if(!wirelessget_fn)
    {
        fprintf(stderr, "Warning: driver does not implement `driver_wireless_get`\n");
        return -1;
    }

    return wirelessget_fn(device->handle, output);
}
