#ifndef DRIVER_INTERFACE_H
#define DRIVER_INTERFACE_H

#include <stdint.h>
#include <stdbool.h>

typedef enum DEVICE_CAPACITY
{
    DEVICE_CAPACITY_MOUSE = (1 << 0),
    DEVICE_CAPACITY_KEYBOARD = (1 << 1),
    DEVICE_CAPACITY_WIRELESS = (1 << 10),
} device_capacity_t;

void driver_init(); // Initialization
uint32_t driver_getkey(); // Calls to this function must return USB keys that this driver registers, until 0

device_capacity_t driver_get_capacity(void* handle); // Returns device capacity (see above)
char* driver_get_name(void* handle); // Returns peripheral name
char* driver_get_manufacturer(void* handle); // Returns peripheral manufacturer
char* driver_get_image(void* handle); // Returns peripheral image

/* Mouse drivers */
struct MOUSE
{
    // DPI
    unsigned int max_level_count;
    unsigned int level_count;
    unsigned int level_current;
    bool led_available;
    bool xy_available;
    unsigned int dpi_x[8];
    unsigned int dpi_y[8];
    unsigned char r[8];
    unsigned char g[8];
    unsigned char b[8];

    // Polling rate
    unsigned int polling_rate_count;
    unsigned int polling_rate_current;
    unsigned int polling_rates[8];

    // Features
    bool angle_snap;
    bool motion_sync;
};
int driver_mouse_get(void* handle, struct MOUSE* output);
int driver_mouse_set(void* handle, struct MOUSE* input);

/* Wireless drivers */
struct WIRELESS
{
    int battery_level;
    bool charging;
    bool connection_wireless;
};
int driver_wireless_get(void* handle, struct WIRELESS* output);

#endif