[CCode (cheader_filename = "utils/array.h")]
namespace Array
{
    void* get(void* array, size_t index);
    size_t count(void* array);
}

[CCode (cheader_filename = "device/device.h")]
namespace Device
{
    // Obtain device array
    void* get_array();

    [CCode (cname = "device_capacity_t", cprefix = "DEVICE_CAPACITY_", has_type_id = false)]
    public enum Capacity
    {
        MOUSE = (1 << 0),
        KEYBOARD = (1 << 1),
        WIRELESS = (1 << 10)
    }

    // Handle a specific device
    unowned string get_name(void* device);
    string get_image(void* device);
    int get_capacity(void* device);
    unowned string get_manufacturer(void* device);

    [CCode (cname = "struct MOUSE", free_function = "", has_type_id = false)]
    public struct Mouse
    {
        // DPI
        uint max_level_count;
        uint level_count;
        uint level_current;
        bool led_available;
        bool xy_available;
        uint dpi_x[8];
        uchar r[8];
        uchar g[8];
        uchar b[8];
    
        // Polling rate
        uint polling_rate_count;
        uint polling_rate_current;
        uint polling_rates[8];
    
        // Features
        bool angle_snap;
        bool motion_sync;
    }

    public int mouse_get(void* device, Mouse* output);
    public int mouse_set(void* device, Mouse* input);

    [CCode (cname = "struct WIRELESS", free_function = "", has_type_id = false)]
    public struct Wireless
    {
        int battery_level;
        bool charging;
        bool connection_wireless;
    }

    public int wireless_get(void* device, Wireless* output);
    public int wireless_set(void* device, Wireless* input);
}
