using Device;

[GtkTemplate (ui="/v/ginput/panel-mouse.ui")]
public class MousePanel : Panel
{
    // Mouse presentation
    [GtkChild]
    private unowned Gtk.Image mouse_image;
    [GtkChild]
    private unowned Gtk.Label mouse_manufacturer;
    [GtkChild]
    private unowned Gtk.Label mouse_name;

    // Wireless informations
    [GtkChild]
    private unowned Adw.PreferencesGroup wireless;
    [GtkChild]
    private unowned Gtk.LevelBar battery_level;
    [GtkChild]
    private unowned Gtk.Label battery_level_label;
    [GtkChild]
    private unowned Gtk.Label charging_state;

    // DPI
    [GtkChild]
    private unowned Adw.PreferencesGroup dpi_preference_group;
    [GtkChild]
    private unowned Adw.ButtonContent dpi_stage_add_button;
    
    // Features
    [GtkChild]
    private unowned Adw.SwitchRow motion_sync_switchrow;
    [GtkChild]
    private unowned Adw.SwitchRow angle_snap_switchrow;

    // Polling rate
    [GtkChild]
    private unowned Adw.ComboRow polling_rate_comborow;

    private void* device;

    public MousePanel(string name, void* device)
    {
        base(name);
        this.device = device;

        refresh_device();
    }

    public void refresh_device()
    {
        // Obtain and fill basic information
        mouse_image.set_from_file(Device.get_image(this.device));
        mouse_name.set_label(name);
        mouse_manufacturer.set_label(Device.get_manufacturer(this.device));
        
        // Obtain mouse information
        Mouse mouse = Mouse();
        Device.mouse_get(this.device, &mouse);

        /* Fill information */

        // DPI stages
        string dpi_used_max = "%u/%u".printf(mouse.level_count, mouse.max_level_count);
        dpi_stage_add_button.set_label(dpi_used_max);

        // All dpi levels
        Gtk.CheckButton? group = null;
        for(size_t i = 0; i < mouse.level_count; i++)
        {
            Adw.ActionRow row = new Adw.ActionRow();

            // Add radiobutton
            if(mouse.level_count > 1)
            {
                Gtk.CheckButton radio_button = new Gtk.CheckButton();
                if(group == null) group = radio_button;
                else radio_button.set_group(group);

                if(mouse.level_current == i)
                    radio_button.set_active(true);
                
                row.add_prefix(radio_button);
            }

            row.set_title("%u DPI".printf(mouse.dpi_x[i]));

            dpi_preference_group.add(row);
        }

        // Features
        motion_sync_switchrow.set_active(mouse.motion_sync);
        angle_snap_switchrow.set_active(mouse.angle_snap);

        // Polling rate
        var model = new Gtk.StringList(null);
        for(size_t i = 0; i < mouse.polling_rate_count; i++)
        {
            model.append("%u Hz".printf(mouse.polling_rates[i]));
        }
        polling_rate_comborow.set_model(model);
        polling_rate_comborow.set_selected(mouse.polling_rate_current);

        // Wireless
        if((Device.get_capacity(this.device) & Device.Capacity.WIRELESS) != 0)
        {
            Wireless wireless = Wireless();
            Device.wireless_get(this.device, &wireless);

            battery_level.set_value(wireless.battery_level);
            battery_level_label.set_label("%u %%".printf(wireless.battery_level));
            if(wireless.charging)
                charging_state.set_label("Charging");
            else
                charging_state.set_label("Not charging");
        }
        else
        {
            wireless.hide();
        }
    }
}
